#ifndef sp2_brainbug_h__
#define sp2_brainbug_h__

#include "cbase.h"
#include "datacache/imdlcache.h"
#include "monstermaker.h"
#include "sp2_bugpoi.h"
#include "EntityList.h"
#include "entitylist_base.h"
#include "sp2_bug.h"
#include "entitylist_base.h"
#include "ai_squad.h"
#include "sp2_round.h"

class CBugNPC;
class BrainBugEntity : public CNPCMaker
{
public:
	DECLARE_CLASS( BrainBugEntity, CNPCMaker );
	
	DECLARE_DATADESC();

	// Constructor
	BrainBugEntity();

	void RefreshEntities( void );

	void Spawn( void );
	void RoundThink( void );
	void FirstThink( void );
	void SpawnThink( void );
	void KillThePlayer( void );
	
	void MakeAnt( bool special, bool run );
	void MakeTurret();
	
	void UpdateConVars( void );

	// Tell us when a POI has been reached by a bug
	void ReachedPOI( CBugNPC* bug, BugPOI* poi );

	void AntKilled( CBugNPC* bug );

	// Tell us when one of our antlions have actually died
	void SpawnAnt( inputdata_t &inputData );

	const char * objective_name;
	const char * objective_description;
	int objective_max_lives;
	int objective_lives_taken;

	COutputEvent m_OnBeginRound;
	COutputEvent m_OnSpawnTurret;
	COutputEvent m_OnBugWin;
	COutputEvent m_OnPlayerWin;
	COutputEvent m_OnBugReachedPOI;

private:
	// The first POI in the level
	string_t m_sFirstPOI;

	// How many bugs have reached the current progress POI?
	int m_nCurrentCount;

	// Which points of interest do we have? (filled up on map load)
	// We have a local cache of these items, because looking up entities in a map is O(n)
	// This is still O(n) but on a vastly smaller list
	CUtlVector<BugPOI*> poiList, pCurrentSpawnPoints;

	// Our Rounds
	CUtlVector<CUtlVector<Sp2Round*> > rounds;
	int roundIndex;
	int spawnIndex;


	// Used internally to figure out where best to spawn the antlions
	BugPOI *targetPOI;
	BugPOI *firstPOI;

	Sp2Round *currentRound;

	void SetFirstPOI();
	
	BugPOI* SelectSpawnPOI();
	void FindSpawnPositions();
	void FindNextTargetPOI();

	void PlayerLost();
	void PlayerWon( void );
};
#endif // sp2_brainbug_h__
