#include "hud.h"
#include "cbase.h"
#include "sp2_objective.h"
#include "iclientmode.h"
#include "hud_macros.h"
#include "vgui_controls/controls.h"
#include "vgui/ISurface.h"
#include "sdk_ourrules.h"

#include "tier0/memdbgon.h"

using namespace vgui;

DECLARE_HUDELEMENT( sp2_objective );

sp2_objective::sp2_objective( const char *pElementName ) : CHudElement( pElementName ), BaseClass( NULL, "sp2_objective" )
{
	Panel *pParent = g_pClientMode->GetViewport();
	SetParent( pParent );   

	SetHiddenBits( HIDEHUD_PLAYERDEAD | HIDEHUD_NEEDSUIT );

	pScheme = scheme()->GetIScheme(GetScheme());

	captionFont = pScheme->GetFont( "HudNumbers" );
	captionColor = pScheme->GetColor( "ObjCaption", Color(255, 208, 64, 255) );
	descFont = pScheme->GetFont( "HudNumbers" );
	descColor = pScheme->GetColor( "ObjDescription", Color(255, 208, 64, 255) );

	fullColor = pScheme->GetColor( "ObjProgressBg", Color( 255, 160, 0, 42 ) );
	lifeColor = pScheme->GetColor( "ObjProgressLife", Color( 28, 107, 160, 255 ) ); // Blood orange, halved
	deathColor = pScheme->GetColor( "ObjProgressFg", Color( 204, 17, 0, 255 ) ); // Blood orange

	objTitleLabel = new Label(this, "objTitleLabel", "");
	objTitleLabel->SetPaintBackgroundEnabled(false);
	objTitleLabel->SetPaintBorderEnabled(false);
	objTitleLabel->SetFont(captionFont);

	healthTitleLabel = new Label(this, "healthTitleLabel", "");
	healthTitleLabel->SetPaintBackgroundEnabled(false);
	healthTitleLabel->SetPaintBorderEnabled( false );
	healthTitleLabel->SetFont(descFont);

	currentObjectiveName = new char;
}

sp2_objective::~sp2_objective(void)
{
	delete(currentObjectiveName);
	delete(objTitleLabel);
	delete(healthTitleLabel);
}

void sp2_objective::Paint()
{
	BaseClass::Paint();

	// Objective health bar background
	surface()->DrawSetColor(fullColor);
	surface()->DrawFilledRect(
		scheme()->GetProportionalScaledValue(objHPxpos + 9), 
		scheme()->GetProportionalScaledValue(24),
		scheme()->GetProportionalScaledValue(objHPxpos + 211), 
		scheme()->GetProportionalScaledValue(30));

	// Player health bar background
	surface()->DrawFilledRect(
		scheme()->GetProportionalScaledValue(9), 
		scheme()->GetProportionalScaledValue(24),
		scheme()->GetProportionalScaledValue(211), 
		scheme()->GetProportionalScaledValue(30));

	// Objective health bar life
	surface()->DrawSetColor(lifeColor);
	surface()->DrawFilledRect(
		scheme()->GetProportionalScaledValue(objHPxpos + 10), 
		scheme()->GetProportionalScaledValue(25),
		scheme()->GetProportionalScaledValue(objHPxpos + 210), 
		scheme()->GetProportionalScaledValue(29));

	// Player health bar life
	surface()->DrawFilledRect(
		scheme()->GetProportionalScaledValue(10), 
		scheme()->GetProportionalScaledValue(25),
		scheme()->GetProportionalScaledValue(210), 
		scheme()->GetProportionalScaledValue(29));


	// Objective health bar deathness
	surface()->DrawSetColor(deathColor);
	deathness = 200 * sp2_objective_lives_taken.GetInt() / sp2_objective_max_lives.GetInt();
	if(deathness > 200)
		deathness = 200;
	surface()->DrawFilledRect(
		scheme()->GetProportionalScaledValue((objHPxpos + 210) - deathness),
		scheme()->GetProportionalScaledValue(25),
		scheme()->GetProportionalScaledValue(objHPxpos + 210),
		scheme()->GetProportionalScaledValue(29));

	// Player health bar deathness
	deathness = 0;
	float lifeFraction = 1.0f;
	if(pPlayer != NULL)
		lifeFraction = (float)pPlayer->GetHealth() / 100;
	deathness = 200 * (1.0f - lifeFraction);
	if(deathness > 200)
		deathness = 200;
	surface()->DrawFilledRect(
		scheme()->GetProportionalScaledValue(210 - deathness),
		scheme()->GetProportionalScaledValue(25),
		scheme()->GetProportionalScaledValue(210),
		scheme()->GetProportionalScaledValue(29));
}

void sp2_objective::OnThink()
{
	objTitleLabel->SetText("Objective health");
	objTitleLabel->SizeToContents();
	objTitleLabel->SetPos(scheme()->GetProportionalScaledValue(637) - objTitleLabel->GetWide(), 0);
	objTitleLabel->SetWide(scheme()->GetProportionalScaledValue(640));
	objTitleLabel->SetFgColor(captionColor);

	healthTitleLabel->SetText("Your health");
	healthTitleLabel->SetPos(scheme()->GetProportionalScaledValue(3), 0);
	healthTitleLabel->SizeToContents();
	healthTitleLabel->SetWide(scheme()->GetProportionalScaledValue(640));
	healthTitleLabel->SetFgColor(descColor);

	pPlayer = C_BasePlayer::GetLocalPlayer();

	BaseClass::OnThink();
}