#include "cbase.h"
#include "ai_network.h"
#include "ai_networkmanager.h"
#include "sdk_learningnodeweight.h"
// memdbgon must be the last include file in a .cpp file!!!
#include "tier0/memdbgon.h"

ConVar nodeweight_learning_sample_size( "nodeweight_learning_sample_size", "30");

LINK_ENTITY_TO_CLASS( sdk_learningnodeweight, CLearningNodeWeight );

// Start of our data description for the class
BEGIN_DATADESC( CLearningNodeWeight )
	
END_DATADESC()

void CLearningNodeWeight::Initialize()
{
	m_fNumberOfUpdates = 0.0;
	Buffer = CUtlVector<Vector>(0, g_pBigAINet->NumNodes());
	BaseClass::Initialize();
}


// Updates the weights over time
void CLearningNodeWeight::Update()
{
	float updateFactor = 1 / (m_fNumberOfUpdates + 1);
	Vector resetValue = Vector(0.0,0.0,0.0);

	for(int i=CurrentWeights.Count()-1;i>=0;i--) 
	{
		Buffer[i] = (CurrentWeights[i] + Buffer[i] * m_fNumberOfUpdates) * updateFactor;
		CurrentWeights[i] = resetValue;
	}

	m_fNumberOfUpdates += 1.0;
	m_fNumberOfUpdates *= (1 - 1/nodeweight_learning_sample_size.GetFloat());

	//DevMsg("Updated nodeweights. NumberOfUpdates is now %f\n", m_fNumberOfUpdates);
}

Vector CLearningNodeWeight::GetNodeWeight(int nodeIndex) 
{
	return (CurrentWeights[nodeIndex] + Buffer[nodeIndex] * m_fNumberOfUpdates) / (m_fNumberOfUpdates + 1);
}

void CLearningNodeWeight::ChangeWeight(int nodeIndex, Vector amount) 
{
	CurrentWeights[nodeIndex] += amount;
}

void CLearningNodeWeight::SetWeight(int nodeIndex, Vector amount) 
{
	CurrentWeights[nodeIndex] = amount;
}



void CLearningNodeWeight::Reset()
{
	BaseClass::Reset();

	Buffer.RemoveAll();

	Vector a = Vector(0.0,0.0,0.0);
	for(int i=0;i<g_pBigAINet->NumNodes();i++) 
	{
		Buffer.AddToTail(a);
	}
}
