#include "cbase.h"
#include "sdk_nodeweight.h"
#include "ai_network.h"
#include "ai_networkmanager.h"
// memdbgon must be the last include file in a .cpp file!!!
#include "tier0/memdbgon.h"

ConVar node_update_interval( "node_update_interval", "5.0", FCVAR_ARCHIVE | FCVAR_CHEAT, "How often to update the nodes." );
ConVar sp2_pause_updates( "sp2_pause_updates", "0", FCVAR_CHEAT, "How often to update the nodes." );

LINK_ENTITY_TO_CLASS( sdk_nodeweight, CNodeWeight );

// Start of our data description for the class
BEGIN_DATADESC( CNodeWeight )
	DEFINE_THINKFUNC( UpdateThink ),
END_DATADESC()

void CNodeWeight::Initialize() 
{
	CurrentWeights = CUtlVector<Vector>(0, g_pBigAINet->NumNodes());
	this->Reset();	
	
	SetThink ( &CNodeWeight::UpdateThink );
	SetNextThink( gpGlobals->curtime + node_update_interval.GetFloat() );
}


void CNodeWeight::UpdateThink()
{
	if(!sp2_pause_updates.GetBool())
		this->Update();
	
	SetNextThink( gpGlobals->curtime + node_update_interval.GetFloat() );
}

void CNodeWeight::Reset() 
{
	DevMsg("Resetting Node Weights\n");
	CurrentWeights.RemoveAll();

	Vector a = Vector(0.0,0.0,0.0);
	for(int i=0;i<g_pBigAINet->NumNodes();i++) 
	{
		CurrentWeights.AddToTail(a);
	}
}

int CNodeWeight::Count() 
{
	return CurrentWeights.Count();
}

Vector CNodeWeight::GetNodeWeight(int nodeIndex) 
{
	return CurrentWeights[nodeIndex];
}

void CNodeWeight::ChangeWeight(int nodeIndex, Vector amount) 
{
	CurrentWeights[nodeIndex] += amount;
}

void CNodeWeight::SetWeight(int nodeIndex, Vector amount) 
{
	CurrentWeights[nodeIndex] = amount;
}
