#ifndef sp2_bug_h__
#define sp2_bug_h__

#include "cbase.h"
#include "ai_default.h"
#include "ai_task.h"
#include "ai_schedule.h"
#include "ai_hull.h"
#include "ai_network.h"
#include "ai_networkmanager.h"
#include "ai_node.h"
#include "ai_link.h"
#include "soundent.h"
#include "game.h"
#include "npcevent.h"
#include "entitylist.h"
#include "activitylist.h"
#include "ai_basenpc.h"
#include "npc_antlion.h"
#include "hl2_shareddefs.h"
#include "engine/IEngineSound.h"
#include "physics_prop_ragdoll.h"
#include "sdk_ourrules.h"
#include "sp2_bugpoi.h"
#include "sp2_brainbug.h"


//=========================================================
// Custom schedules
//=========================================================
enum
{
	SCHED_RUN_TO_GOAL = LAST_SHARED_SCHEDULE,
	SCHED_WALK_TO_GOAL,
	SCHED_BUG_IDLE,
	SCHED_BUG_BURROW,
	SCHED_BUG_UNBURROW,
	SCHED_BUG_BURROWED,
};

//=========================================================
// Custom tasks
//=========================================================
enum 
{
	TASK_FIND_BUG_GOAL = LAST_SHARED_TASK,
	TASK_STAY_BURROWED

};


//=========================================================
// Custom Conditions
//=========================================================
enum 
{
	COND_BUG_REACHED_GOAL = LAST_SHARED_CONDITION,
};


//=========================================================
//=========================================================
class BrainBugEntity;
class CBugNPC : public CAI_BaseNPC
{
	DECLARE_CLASS( CBugNPC, CAI_BaseNPC );

public:
	void	Precache( void );
	void	Spawn( void );
	Class_T Classify( void );

	DECLARE_DATADESC();

	HSOUNDSCRIPTHANDLE	m_hFootstep;

	int SelectSchedule( );
	void HandleAnimEvent( animevent_t *pEvent );
	int OnTakeDamage_Alive( const CTakeDamageInfo &info );
	void OnFriendDamaged( CBaseCombatCharacter *pSquadmate, const CTakeDamageInfo &info );

	void SetBurrowed(bool value);

	bool MovementCost(int movetype, const Vector &vecStart, const Vector &vecEnd, float *pCost, CAI_Node* start = NULL, CAI_Node* end = NULL);

	void StartTask( const Task_t *pTask );
	void RunTask( const Task_t *pTask );
	void GatherConditions();
	bool ShouldGib( const CTakeDamageInfo &info ) ;	// Always ragdoll, unless specified by the leaf class
	void SetTargetPOI( BugPOI *targetPOI );
	bool CorpseGib( const CTakeDamageInfo &info );

	bool FindChasePosition( const Vector &targetPos, Vector &result );
	bool GetGroundPosition( const Vector &testPos, Vector &result );
	void BuildScheduleTestBits( void );

	int MeleeAttack1Conditions( float flDot, float flDist );
	void MeleeAttack( float distance, float damage, QAngle &viewPunch, Vector &shove );
	void Burrow( void );
	void Unburrow( void );
	void CreateDust( bool placeDecal = true );
	void Event_Killed( const CTakeDamageInfo &info );
	void IdleSound( void );
	void PainSound( const CTakeDamageInfo &info );

	Vector m_vecArmorColor;
	Vector m_vecCostFactor;
	Vector m_vecAccumulatedDamage;

	bool m_bIsBurrowed;
	bool m_bCanRun;

	BugPOI *m_eTargetPOI;
	BrainBugEntity *m_eBRAAIN;

	DEFINE_CUSTOM_AI;
};
#endif // sp2_bug_h__