#include "cbase.h"
#include "ai_network.h"
#include "ai_networkmanager.h"
#include "ai_debug.h"
#include "sp2_bug.h"
#include "sp2_bugpoi.h"
// memdbgon must be the last include file in a .cpp file!!!
#include "tier0/memdbgon.h"








// Console commands for changing the weights of nodes
ConVar node_weight_adjust_amount( "node_weight_adjust_amount", "0.25", FCVAR_ARCHIVE | FCVAR_CHEAT, "How much to adjust node weights using node_weight_inc and node_weight_dec" );


void AdjustWeight( float amount ) 
{
	CBasePlayer* pPlayer = UTIL_GetCommandClient();
	trace_t tr;
	Vector forward, v;
	pPlayer->EyeVectors( &forward );
	AI_TraceLine(pPlayer->EyePosition(),
		pPlayer->EyePosition() + forward * MAX_TRACE_LENGTH,MASK_NPCSOLID, 
		pPlayer, COLLISION_GROUP_NONE, &tr );

	if(tr.fraction < 1.0)
	{
		v = tr.endpos;
		CAI_Network*		ai = g_pBigAINet;
		
		int i = ai->NearestNodeToPoint(v, false);

		
		CNodeWeight* weighter = g_pAINetworkManager->m_pNodeWeight;
		
		if(i >= 0 && i < weighter->Count()) 
		{
			weighter->ChangeWeight(i, Vector(amount, amount, amount));
		}
	}
}

void IncreaseWeight() { AdjustWeight(node_weight_adjust_amount.GetFloat()); }
void DecreaseWeight() { AdjustWeight(-node_weight_adjust_amount.GetFloat()); }

static ConCommand node_weight_inc("node_weight_inc", IncreaseWeight, "Gap", FCVAR_CHEAT);
static ConCommand node_weight_dec("node_weight_dec", DecreaseWeight, "Gap", FCVAR_CHEAT);


void UpdateNodeWeights()
{
	int times = 1;
	if(	engine->Cmd_Argc() > 1 ) 
	{
		times = atoi(engine->Cmd_Argv(1));
	}
	Msg("Updating nodeweights %i times\n", times);

	CNodeWeight* weighter = g_pAINetworkManager->m_pNodeWeight;

	for(int i=0;i<times;i++) 
	{
		weighter->Update();
	}
}

static ConCommand update_node_weights("update_node_weights", UpdateNodeWeights, "Calls the update function of the nodeweights subsystem");




// TEST


void CC_SP2_Route( void )
{
	/*CBasePlayer* pPlayer = UTIL_GetCommandClient();
	trace_t tr;
	Vector forward;
	pPlayer->EyeVectors( &forward );
	AI_TraceLine(pPlayer->EyePosition(),
		pPlayer->EyePosition() + forward * MAX_TRACE_LENGTH,MASK_NPCSOLID, 
		pPlayer, COLLISION_GROUP_NONE, &tr );
	*/
	
	Vector point = Vector(0,0,0);
	CBaseEntity * pp = NULL;
	BugPOI* pBugPoi = NULL;
	while((pp = gEntList.FindEntityByClassname(pp, "sp2_bugpoi")) !=  NULL)
	{
		pBugPoi = (BugPOI*)pp;
		if(pBugPoi->m_nTypeOfPOI == 1) 
		{
			point = pBugPoi->GetAbsOrigin();
			break;
		}
	}

	while((pp = gEntList.FindEntityByClassname(pp, "sp2_bug")) !=  NULL)
	{
		CBugNPC* pBug = (CBugNPC*)pp;
		pBug->m_vecArmorColor = Vector(1,1,0);
		pBug->m_vecCostFactor = Vector(1,1,0);	
		pBug->TranslateNavGoal( pBugPoi, point );
		pBug->GetNavigator()->SetGoal( point );
		pBug->m_debugOverlays |= OVERLAY_NPC_ROUTE_BIT;
	}
}
static ConCommand sp2_route("sp2_route", CC_SP2_Route, "", FCVAR_CHEAT);

extern ConVar sp2_splash_damage_range;

void CC_Add_Splash_Weight( void )
{
	CBasePlayer* pPlayer = UTIL_GetCommandClient();
	trace_t tr;
	Vector forward, v;
	pPlayer->EyeVectors( &forward );
	AI_TraceLine(pPlayer->EyePosition(),
		pPlayer->EyePosition() + forward * MAX_TRACE_LENGTH,MASK_NPCSOLID, 
		pPlayer, COLLISION_GROUP_NONE, &tr );

	if(tr.fraction < 1.0)
	{
		v = tr.endpos;
		CAI_Network*		ai = g_pBigAINet;

		int nodeId = ai->NearestNodeToPoint(v, false);

		CNodeWeight* weighter = g_pAINetworkManager->m_pNodeWeight;
		
		if(nodeId >= 0 && nodeId < weighter->Count()) 
		{
			// add red danger (player)
			Vector dangerChange = Vector(1,0,0);

			// Maximum register 1 death each time damage is dealt
			dangerChange.x = min(dangerChange.x, 1);
			dangerChange.y = min(dangerChange.y, 1);
			dangerChange.z = min(dangerChange.z, 1);

			weighter->ChangeWeight(nodeId, dangerChange);

			float range = sp2_splash_damage_range.GetFloat();

			// Give neighbour nodes damage according to their distance to the current node.
			CAI_Node* node =  ai->GetNode(nodeId);
			for(int i=0;i<node->NumLinks();i++) 
			{
				CAI_Link* link =  node->GetLinkByIndex(i);
				CAI_Node* destNode = ai->GetNode(link->m_iDestID);
				float dist = (node->GetOrigin() - destNode->GetOrigin()).Length();
				if(dist < range) 
				{
					float distFactor = 1 - (dist / range);
					weighter->ChangeWeight(link->m_iDestID, dangerChange * distFactor);
				}
			}
		}
	}
}

static ConCommand sp2_add_weight("sp2_add_weight", CC_Add_Splash_Weight, "", FCVAR_CHEAT);
